'
' datafolder.bi
'
' FreeBASIC library for "data folders" - normally compressed folders containing application data
' allows to access an uncompressed folder as an identically named compressed archive file
' for example, a folder might be named "myproject.rsc", and all of its content will be
' packed into a ZIP-file that is renamed "myproject.rsc". Using the datafolder library,
' you can treat both identically.
'
' The library determines automatically whether you are having the uncompressed folder,
' or a compressed archive, and determines the type of archive/compression automatically.
'
' You can't change/edit the data files, resp. only with outside tools, but it is only
' intended for static program data anyway.
'
' IMPORTANT: "filepath" is consisting of 2 parts:
'               - the file system path locating the data folder
'               - the internal data folder file path addressing a file inside the
'                 data folder
'               those are divided with a colon (":"), the path may contain e.g. a "A:" or
'               "http://", only the last colon in the path is considered as divider
'
' DFEXISTS(STRING filepath)
'        returns whether specified file is existing (inside data folder)
'
' DFLOAD(STRING filepath, STRING dest_buffer, STRING encryption="") loads the specified
'        data file into the the given destination string, automatically sizing
'
'        encryption is "<algorithm>-><key>", optionally for encrypted files or archives
'        "algorithm" is, for instance:
'          - 'XOR': super-primitive XOR encryption, "key" is a string that is repeatetly
'            XORed with the file content, just makes the content unreadable
'          - 'ZIP': PKZIP "ZIP-Crypto", used by ZIP archives
'          - 'AES': AES-128, AES-192 and AES-256, determined by the key length
'        If the key is contained somewhere in the application package, it won't be safe anyway,
'        just hindering ordinary users to read e.g. texts they shouldn't read (mostly spoilers
'        in computer games' data).
'        Asymmetric encryption might avoid manipulation of application data (also mostly cheating
'        in computer games).
'
' DFLOAD(STRING filepath, BYTE PTR dest_buffer, STRING encryption="")
'        alternate function working with an ALLOCATE-created buffer (demands a DEALLOCATE call)
'

#INCLUDE ONCE "file.bi"

FUNCTION DFISAFOLDER(BYVAL file AS STRING) AS BOOLEAN
  DIM AS INTEGER i

  i=INSTRREV(file, ":")
  IF i THEN
    i-=1
    file=LEFT(file, i)
    DIR(file, &H31, @i)
    IF (i AND &H10) THEN RETURN TRUE ' file type is folder
  ENDIF

  ' #note: returns false, also if file/folder not existing => use DFEXISTS() !
  RETURN FALSE
END FUNCTION


DECLARE SUB DFLOAD OVERLOAD (BYVAL file AS STRING, BYREF fbuffer AS STRING, BYVAL encryption AS STRING="")
SUB DFLOAD(BYVAL file AS STRING, BYREF fbuffer AS STRING, BYVAL encryption AS STRING="")
  DIM AS INTEGER i, l, fn

  i=INSTRREV(file, ":")
  ' #note: / #todo:
  ' a path containing ":" like "https://", "/C:/" will cause trouble, if
  ' the datafolder isn't specified correctly: "...<datafolder>:...[/...[/...]]"
  ' some might use the datafolder library unapparently through other libraries
  ' - at least a path like "C:\" should be translated into unix-like "/C/" or
  ' "/volume/C/" for instance. Common protocols have to be treated specially
  ' anyway
  ' #todo: network / HTTP and HTTPS support

  IF i=-1 OR DFISAFOLDER(file) THEN ' normal, direct file access
    ' "correcting" the path
    IF i<>-1 THEN
      i-=1
      file[i]=47 ' '/'
    ENDIF

    ' loading file in one go
    fn=FREEFILE()
    IF OPEN (file FOR BINARY AS #fn)=0 THEN
      l=LOF(fn)
      fbuffer=SPACE(l)
      GET #fn, 1, fbuffer
      CLOSE #fn
    ENDIF
  ELSE ' accessing an archive file
    ' #todo:
    ' - determine (archive) file format, normally ZIP, but anything goes in principle
    ' - open archive, get specified file, unpack its content into the buffer
  ENDIF
END SUB


SUB DFLOAD(BYVAL file AS STRING, BYREF fbuffer AS BYTE PTR, BYVAL encryption AS STRING="")
  ' it's like ordinary DFLOAD but with an ALLOCATE-buffer,
  ' automatically allocating if necessary
  ' #todo: - it's not implemented...
END SUB


FUNCTION DFEXISTS(BYVAL file AS STRING) AS BOOLEAN
  DIM AS INTEGER i

  i=INSTRREV(file, ":")
  IF i THEN
    IF DFISAFOLDER(file) THEN
      ' => check if file exists
      i-=1
      file[i]=47 ' replace : with /
      IF FILEEXISTS(file) THEN RETURN TRUE
    ELSE ' file is a compressed archive
      ' #todo: determine archive format and check...
    ENDIF
  ENDIF

  ' any fail means "doesn't exist"...
  RETURN FALSE
END FUNCTION
